<?php
/**
 * Database Class
 */
class Database {
    
    private $host      = DB_HOST;
    private $user      = DB_USER;
    private $pass      = DB_PASS;
    private $dbname    = DB_NAME;
 
    private $dbh;       // DB Handler var
    private $stmt;      // Statement var
    private $error;     // Error var
 
    /**
     * Class constructor / initializer
     */
    public function __construct($host = null, $user = null, $pass = null, $dbname = null) 
    {
        //If any use database configuration passed as parameters via __construct(...) else use values from DB_* constants
        if (!is_null($host)) {
            $this->host = $host;
        }
        if (!is_null($user)) {
            $this->user = $user;
        }
        if (!is_null($pass)) {
            $this->pass = $pass;
        }
        if (!is_null($dbname)) {
            $this->dbname = $dbname;
        }
        
        // Set DSN
        $dsn = 'mysql:host=' . $this->host . ';dbname=' . $this->dbname;
        // Set options
        $options = array(
            PDO::ATTR_PERSISTENT    => true,
            PDO::ATTR_ERRMODE       => PDO::ERRMODE_EXCEPTION,
			PDO::MYSQL_ATTR_USE_BUFFERED_QUERY => true
        );
        // Create a new PDO instanace
        try {
            $this->dbh = new PDO($dsn, $this->user, $this->pass, $options);
        }
        // Catch any errors
        catch(PDOException $e){
            $this->error = $e->getMessage();
        }
    }
    
    /**
     * Prepare SQL Query Statement 
     * @param string $query : SQL query statment
     */
    public function prepare_query($query)
    {
        $this->stmt = $this->dbh->prepare($query);
    }
    
    /**
     * Bind value to a Parameter
     * @param string $param : is the placeholder value that we will be using in our SQL statement, example :name.
     * @param string $value : is the actual value that we want to bind to the placeholder, example Vic Bob.
     * @param string $type  : is the datatype of the parameter, example string.
     */
    public function bind_value($param, $value, $type = null)
    {
        if (!is_null($type)) {
            switch ($type) {
                case 'int':
                    $type = PDO::PARAM_INT;
                    break;
                case 'bool':
                    $type = PDO::PARAM_BOOL;
                    break;
                case 'null':
                    $type = PDO::PARAM_NULL;
                    break;
                default:
                    $type = PDO::PARAM_STR;
            }
        } else {
            switch (true) {
                case is_int($value):
                    $type = PDO::PARAM_INT;
                    break;
                case is_bool($value):
                    $type = PDO::PARAM_BOOL;
                    break;
                case is_null($value):
                    $type = PDO::PARAM_NULL;
                    break;
                default:
                    $type = PDO::PARAM_STR;
            }
        }
        $this->stmt->bindValue($param, $value, $type);
    }
    
    /**
     * Execute a Query
     */
    public function execute_query($error_msg_prefix_str='', $rollback_transaction=false)
    {
        try {
            return $this->stmt->execute();
            
        } catch (PDOException $e){
            
            echo $error_message = "$error_msg_prefix_str() ==> " . $e->getMessage();
            if ($rollback_transaction === true) {
                $this->rollback_transaction();
            }
            exit;
        }
    }
    
    /**
     * Fetch All Result 
     */
    public function fetch_all_result()
    {
        return $this->stmt->fetchAll(PDO::FETCH_ASSOC);
    }    
    
    /**
     * Fetch Single Results 
     */
    public function fetch_single_result()
    {
        return $this->stmt->fetch(PDO::FETCH_ASSOC);
    }
    
    /**
     * Count Result Row
     */
    public function count_row()
    {
        return $this->stmt->rowCount();
    }
    
    /**
     * Last Insert Id
     */
    public function last_insert_ID()
    {
        return $this->dbh->lastInsertId();
    }
    
    /**
     * Begin a transaction:
     */
    public function begin_transaction()
    {
        return $this->dbh->beginTransaction();
    }
    
    /**
     * End a transaction and commit your changes:
     */
    public function end_transaction()
    {
        return $this->dbh->commit();
    }
    
    /**
     *  Cancel a transaction and roll back your changes:
     */
    public function rollback_transaction()
    {
        return $this->dbh->rollBack();
    }
    
    /**
     * Debug Dump Parameters 
     */
    public function debug_dump_params()
    {
        return $this->stmt->debugDumpParams();
    }
}
?>